/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MgmtServlet.java,v 1.18 2004/08/30 12:51:37 danesa Exp $
 * --------------------------------------------------------------------------
 */

package org.objectweb.j2eemanagement.servlets;

//import java
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.List;

import javax.management.MBeanException;
import javax.management.ObjectName;
import javax.management.j2ee.ListenerRegistration;
import javax.management.j2ee.Management;
import javax.management.j2ee.ManagementHome;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * This servlet is an example which shows how to access the MEJB from a servlet.
 * @author JOnAS team
 * @author Adriana Danes
 * @author Michel-Ange Anton
 */
public class MgmtServlet extends HttpServlet {

    // ---------------------------------------------------------- Constants
    /** Printing constant*/
    static final String APP_TITLE_LARGE = "J2EE Management sample with Servlet accessing the MEJB";
    /** Printing constant*/
    static final String APP_TITLE = "J2EE Management sample";
    /** Parameter */
    static final String PARAM_DOMAIN = "domainName";
    /** Parameter */
    static final String PARAM_VIEW = "view";
    /** Parameter */
    static final String VIEW_INIT = "init";
    /** Parameter */
    static final String VIEW_NOTIFICATIONS = "notifications";
    /** Parameter */
    static final String VIEW_OTHER = "other";
    /**
     * Listener object
     */
    private MyListener mListener = null;

    // ---------------------------------------------------------- Public methods

    /**
     * Initialize the servlet.
     * @param pConfig See HttpServlet
     * @throws ServletException
     */
    public void init(ServletConfig pConfig) throws ServletException {
        super.init(pConfig);
        // Initialize variables
        mListener = null;
    }

    // ---------------------------------------------------------- Protected
    // methods

    /**
     * Response to the GET request.
     * @param pRequest See HttpServlet
     * @param pResponse See HttpServlet
     * @throws IOException
     * @throws ServletException
     */
    protected void doGet(HttpServletRequest pRequest, HttpServletResponse pResponse) throws IOException,
            ServletException {
        dispatch(pRequest, pResponse);
    }

    /**
     * Response to the POST request.
     * @param pRequest See HttpServlet
     * @param pResponse See HttpServlet
     * @throws IOException
     * @throws ServletException
     */
    protected void doPost(HttpServletRequest pRequest, HttpServletResponse pResponse) throws IOException,
            ServletException {
        dispatch(pRequest, pResponse);
    }

    /**
     * Dispatch the response.
     * @param pRequest Request
     * @param pResponse Response
     * @throws IOException
     */
    protected void dispatch(HttpServletRequest pRequest, HttpServletResponse pResponse) throws IOException {

        pResponse.setContentType("text/html");
        PrintWriter out = pResponse.getWriter();

        // Get parameters
        String sParamDomain = pRequest.getParameter(PARAM_DOMAIN);
        String sParamView = pRequest.getParameter(PARAM_VIEW);

        // Dispatching
        if ((sParamDomain == null) || (sParamDomain.length() == 0)) {
            doViewError("Parameter domain not found", pRequest, out);
        } else if ((sParamView == null) || (sParamView.length() == 0) || VIEW_INIT.equals(sParamView)) {
            doViewInit(sParamDomain, pRequest, out);
        } else if (VIEW_NOTIFICATIONS.equals(sParamView)) {
            doViewNotifications(out);
        } else if (VIEW_OTHER.equals(sParamView)) {
            doViewOther(sParamDomain, pRequest, out);
        } else {
            doViewError("Unknown View", pRequest, out);
        }

    }

    /**
     * Header.
     * @param pOut Printer
     * @param pTitle Title to display
     * @param pSubTitle Subtitle to display or null if not
     */
    protected void printHeader(PrintWriter pOut, String pTitle, String pSubTitle) {

        pOut.println("<html>");
        pOut.println("<head>");
        pOut.println("<title>" + pTitle);
        if (pSubTitle != null) {
            pOut.println(" - " + pSubTitle);
        }
        pOut.println("</title>");
        pOut.println("</head>");
        pOut.println("<body style=\"background : white; color : black;\">");
        printHeaderTitle(pOut, pTitle, pSubTitle);
    }

    /**
     * Header.
     * @param pOut Printer
     * @param pTitle Title to display
     * @param pSubTitle Subtitle to display or null if not
     */
    protected void printHeaderAutoRefresh(PrintWriter pOut, String pTitle, String pSubTitle) {

        pOut.println("<html>");
        pOut.println("<head>");
        pOut.println("<title>" + pTitle + "</title>");
        pOut.println("<script language=\"JavaScript\">");
        pOut.println("function startRefresh(){ setTimeout(\"refresh()\",30000); self.focus(); }");
        pOut.println("function refresh(){ window.location.reload(); }");
        pOut.println("</script>");
        pOut.println("</head>");
        pOut.println("<body onload=\"startRefresh()\" style=\"background : white; color : black;\">");
        printHeaderTitle(pOut, pTitle, pSubTitle);
    }

    /**
     * Header title.
     * @param pOut Printer
     * @param pTitle Title to display
     * @param pSubTitle Subtitle to display or null if not
     */
    protected void printHeaderTitle(PrintWriter pOut, String pTitle, String pSubTitle) {

        pOut.println("<table width=\"100%\" border=\"0\">");
        pOut.println("<tr>");
        pOut.println("<td rowspan=\"2\"><img src=\"img/logo_jonas.jpg\" alt=\"JOnAS Logo\"></td>");
        pOut.println("<td align=\"center\"><h1>" + pTitle + "</h1></td>");
        pOut.println("</tr>");
        pOut.println("<tr>");
        if (pSubTitle != null) {
            pOut.println("<td align=\"center\"><h1>" + pSubTitle + "</h1></td>");
        }
        pOut.println("</tr>");
        pOut.println("</table>");
    }

    /**
     * Footer.
     * @param pOut Printer
     */
    protected void printFooter(PrintWriter pOut) {
        pOut.println("</body>");
        pOut.println("</html>");
    }

    /**
     * Footer navigation.
     * @param pDomainName Name of domain
     * @param pRequest Http request
     * @param pOut Printer
     * @param pPrevious Previous view
     * @param pNext Next view
     */
    protected void printNavigationFooter(String pDomainName, HttpServletRequest pRequest, PrintWriter pOut,
            String pPrevious, String pNext) {

        // index
        String sViewIndex = pRequest.getContextPath();
        // Notifications
        String sViewNotifications = pRequest.getRequestURI() + "?" + PARAM_DOMAIN + "=" + pDomainName + "&"
                + PARAM_VIEW + "=" + VIEW_NOTIFICATIONS;
        // Previous View
        String sPrevView = null;
        if (pPrevious != null) {
            sPrevView = pRequest.getRequestURI() + "?" + PARAM_DOMAIN + "=" + pDomainName + "&" + PARAM_VIEW + "="
                    + pPrevious;
        }
        // Next View
        String sNextView = null;
        if (pNext != null) {
            sNextView = pRequest.getRequestURI() + "?" + PARAM_DOMAIN + "=" + pDomainName + "&" + PARAM_VIEW + "="
                    + pNext;
        }

        // Display
        pOut.print("<table border=\"1\" align=\"center\" cellspacing=\"0\" cellpadding=\"5\">");
        pOut.print("<tr>");
        pOut.print("<td align=\"center\"> <a href=\"" + sViewIndex + "\">Index</a> </td>");
        pOut.print("<td align=\"center\"> <a href=\"" + sViewNotifications
                + "\" target=\"ViewNotifications\">Notifications</a> </td>");
        if (sPrevView != null) {
            pOut.println("<td align=\"center\"> <a href=\"" + sPrevView + "\">Previous</a> </td>");
        }
        if (sNextView != null) {
            pOut.println("<td align=\"center\"> <a href=\"" + sNextView + "\">Next</a> </td>");
        }
        pOut.println("</tr></table>");
        // Footer
        printFooter(pOut);
    }

    /**
     * Simply View error.
     * @param pError Message error
     * @param pRequest Http request
     * @param pOut Printer
     */
    protected void doViewError(String pError, HttpServletRequest pRequest, PrintWriter pOut) {

        // Header
        printHeader(pOut, APP_TITLE, "Error");
        // Error message
        pOut.println("<h2>" + pError + "</h2>");
        // Return
        pOut.println("<a href=\"" + pRequest.getContextPath() + "\">Return</a>");
        // Footer
        printFooter(pOut);
    }

    /**
     * View init MEJB and access to MBEans J2EEDomain and J2EEServer.
     * @param pDomainName Name of domain to access
     * @param pRequest Http request
     * @param pOut Printer
     */
    protected void doViewInit(String pDomainName, HttpServletRequest pRequest, PrintWriter pOut) {

        // Header
        printHeader(pOut, APP_TITLE_LARGE, "Init");

        // -----------------------------
        // Get initial context
        // -----------------------------
        pOut.println("<h2>Initial context</h2>");
        pOut.println("<ul>");
        Context initialContext = null;
        try {
            initialContext = new InitialContext();
            pOut.println("<li>Initial context OK</li>");
        } catch (Exception e) {
            pOut.print("<li>Cannot get initial context for JNDI: ");
            pOut.println(e + "</li>");
            pOut.println("</ul>");
            return;
        }
        pOut.println("</ul><br>");

        // -----------------------------
        // Access to the MEJB
        // -----------------------------
        pOut.println("<h2>Create MEJB</h2>");
        pOut.println("<ul>");

        // Connecting to the MEJB home through JNDI
        ManagementHome mgmtHome = null;
        try {
            mgmtHome = (ManagementHome) PortableRemoteObject.narrow(initialContext.lookup("java:comp/env/ejb/MEJB"),
                    ManagementHome.class);
        } catch (Exception e) {
            pOut.println("<li>Cannot lookup java:comp/env/ejb/mgmt/MEJB: " + e + "</li>");
            pOut.println("</ul>");
            return;
        }
        // Management bean creation
        Management mgmt = null;
        try {
            mgmt = mgmtHome.create();
            pOut.println("<li>MEJB created</li>");
        } catch (Exception e) {
            pOut.println("<li>Cannot create MEJB: " + e + "</li>");
            return;
        }
        pOut.println("</ul><br />");

        // ------------------------------
        // Access to the J2EEDomain MBean
        // ------------------------------
        ObjectName onDomain = accessJ2EEDomain(pDomainName, mgmt, pOut);
        if (onDomain == null) {
            return;
        }
        // -----------------------------------------------------
        // Check for J2EEServer MBeans in the current J2EEDomain
        // -----------------------------------------------------
        ObjectName onServer = accessJ2EEServer(onDomain, mgmt, pOut);
        if (onServer == null) {
            return;
        }

        // ------------------------------
        // Access to the selected J2EEServer MBean
        // ------------------------------
        pOut.println("<h2>Try to manage the selected J2EEServer MBean</h2>");
        pOut.println("<ul>");
        pOut.println("<li>Using the J2EEServer Object Name: \"" + onServer.toString() + "\"</li>");

        // Use deployJar, unDeployJar, isJarDeployed management opearations
        // (suppose sb.jar was installed under JONAS_BASE/ejbjars directory)
        String jarFileName = "sb.jar";
        deployJarModule(onServer, mgmt, jarFileName, pOut);

        // Use deployEar, unDeployEar, isEarDeployed management opearations
        // (suppose earsample.ear was installed under JONAS_BASE/apps directory)
        String earFileName = "earsample.ear";
        deployEarModule(onServer, mgmt, earFileName, pOut);

        // (suppose autoload/earsample.ear was installed under JONAS_BASE/apps directory)
        earFileName = "autoload/earsample.ear";
        deployEarModule(onServer, mgmt, earFileName, pOut);

        // Use deployRar, unDeployRar, isRarDeployed
        // (suppose JDBC connection rar files installed under JONAS_BASE/rars directory)
        String rarFileName = "autoload/JOnAS_jdbcCP.rar";
        deployRarModule(onServer, mgmt, rarFileName, pOut);

        pOut.println("</ul><br>");

        // -----------------------------
        // Test Event Listener support
        // -----------------------------

        pOut.println("<h2>Register a event listener</h2>");
        pOut.println("<ul>");
        if (mListener == null) {
            mListener = new MyListener();
            pOut.println("<li>MyListener created</li>");
        }

        // Get the ListenerRegistration object
        try {
            ListenerRegistration lr = mgmt.getListenerRegistry();
            if (lr != null) {
                pOut.println("<li>Add listener on J2EEServer (" + onServer.toString() + ")</li>");
                String sHandler = "MEJBTester";
                lr.addNotificationListener(onServer, mListener, null, sHandler);
                pOut.println("<li>Notification Listener added</li>");
                String sViewNotifications = pRequest.getRequestURI() + "?" + PARAM_DOMAIN + "=" + pDomainName + "&"
                + PARAM_VIEW + "=" + VIEW_NOTIFICATIONS;
                pOut.println("<a href=\"" + sViewNotifications
                        + "\" target=\"ViewNotifications\">See list of notifications</a>");
                pOut.println("</ul>");
            } else {
                pOut.println("<li>Can't add remote listener for the moment</li>");
                pOut.println("</ul>");
            }
        } catch (Exception e) {
            pOut.println("<li>Can't add notification listener on " + onServer.toString() + " : " + e + "</li>");
        }

        pOut.println("<h2>View Sample Init is OK </h2>");

        // Footer
        printNavigationFooter(pDomainName, pRequest, pOut, null, VIEW_OTHER);
    }

    /**
     * View notifications.
     * @param pOut Printer
     */
    protected void doViewNotifications(PrintWriter pOut) {

        // Header
        printHeaderAutoRefresh(pOut, APP_TITLE, "Notifications");
        pOut.println("<h2>List of notifications</h2>");

        // Verify listener
        if (mListener == null) {
            pOut.println("Listener not found !");
        } else {
            // Display notifications
            List list = mListener.getlistNotifications();
            if (list.size() > 0) {
                pOut.println("<ol>");
                for (int i = 0; i < list.size(); i++) {
                    pOut.println("<li>" + list.get(i) + "</li>");
                }
                pOut.println("</ol>");
            } else {
                pOut.println("No notifications, the list is empty");
            }
        }

        // Footer
        printFooter(pOut);
    }

    /**
     * View example.
     * @param pDomainName Name of domain to access
     * @param pRequest Http request
     * @param pOut Printer
     */
    protected void doViewOther(String pDomainName, HttpServletRequest pRequest, PrintWriter pOut) {

        // Header
        printHeader(pOut, APP_TITLE, "Other");

        pOut.println("<h2>Other</h2>");

        // Footer
        printNavigationFooter(pDomainName, pRequest, pOut, VIEW_INIT, null);
    }

    /**
     * Create J2EEDomain MBean's ObjectName and test if MBean registered
     * @param pDomainName the name provided by the user
     * @param mgmt MEJB
     * @param pOut output stream
     * @return true if management operation succeeded
     */
    private ObjectName accessJ2EEDomain(String pDomainName, Management mgmt, PrintWriter pOut) {
        ObjectName onDomain = null;
        pOut.println("<h2>Access the J2EEDomain MBean</h2>");
        pOut.println("<ul>");

        // Get the J2EEDomain MBean's ObjectName
        try {
            String name = pDomainName + ":j2eeType=J2EEDomain,name=" + pDomainName;
            onDomain = ObjectName.getInstance(name);
            if (onDomain != null) {
                pOut.println("<li>J2EEDomain ObjectName: \"" + name.toString() + "\" created</li>");
            } else {
                pOut.println("<li>J2EEDomain ObjectName not found</li>");
                pOut.println("</ul>");
                return onDomain;
            }
        } catch (Exception e) {
            pOut.println("<li>Cannot create ObjectName for J2EEDomain managed object: " + e + "</li>");
            pOut.println("</ul>");
            return null;
        }
        // Check that the J2EEDomain MBean registered
        try {
            boolean exists = mgmt.isRegistered(onDomain);
            if (exists) {
                pOut.println("<li>This J2EEDomain managed object is registered in the current MBean server</li>");
                pOut.println("</ul>");
            } else {
                pOut.println("<li><b>Can't find the J2EEDomain managed object having the above name in the current MBean server</b></li>");
                pOut.println("</ul>");
                return null;
            }
        } catch (Exception e) {
            pOut.println("<li>Error when using the J2EEDomain managed object: " + e + "</li>");
            pOut.println("</ul>");
            return null;
        }
        return onDomain;
    }

    /**
     * Get a registered J2EEServer MBean
     * @param onDomain J2EEDomain MBean's ObjectName
     * @param mgmt MEJB
     * @param pOut output stream
     * @return true if management operation succeeded
     */
    private ObjectName accessJ2EEServer(ObjectName onDomain, Management mgmt, PrintWriter pOut) {
        pOut.println("<h2>Access the J2EEServer MBean</h2>");
        pOut.println("<ul>");
        ObjectName onServer = null;
        try {
            String[] listServers = (String[]) mgmt.getAttribute(onDomain, "servers");
            int nbServers = listServers.length;
            if (nbServers == 0) {
                pOut.println("<li>No J2EE servers running in this domain.</li>");
                pOut.println("</ul><br>");
                return null;
            } else {
                if (nbServers == 1) {
                    String serverName = (String) listServers[0];
                    onServer = ObjectName.getInstance(serverName);
                    if (mgmt.isRegistered(onServer)) {
                        pOut.println("<li>The J2EE server running in this domain has ObjectName: \"" + serverName + "\" </li>");
                        pOut.println("</ul><br>");
                        return onServer;
                    } else {
                        pOut.println("<li><b>Can't find the J2EEServer managed object having name: " + serverName + " in the current MBean server</b></li>");
                        pOut.println("</ul><br>");
                        return null;
                    }
                } else {
                    pOut.println("<li>List of J2EE servers in this domain (list of associated MBeans):</li>");
                    pOut.println("<ol>");
                    for (int i = 0; i < nbServers; i++) {
                        String sServer = listServers[i];
                        ObjectName on = ObjectName.getInstance(sServer);
                        if (mgmt.isRegistered(on)) {
                            pOut.print("<li>The MBean having object name \"" + sServer);
                            pOut.println("\" is registered in the MBean server.");
                            pOut.println("</li>");
                            onServer = on;
                        } else {
                            pOut.print("<li>Found MBean object name \"" + sServer);
                            pOut.println("\" in the severs list, but this name is not registered in the MBean server.");
                            pOut.println("</li>");
                        }
                    }
                    pOut.println("</ol>");
                    pOut.println("</ul><br>");
                    return onServer;
                }
            }
        } catch (Exception e) {
            pOut.println("<li>Error when managing the J2EEServer: " + e + "</li>");
            pOut.println("</ul><br>");
            return null;
        }
    }
    /**
     * Deploy if not already deployed, undeploy otherwise, a given jar file.
     * @param onServer ObjectName of the J2EEServer MBean on which the mananagement operation has to be applied
     * @param mgmt MEJB
     * @param jarFileName name of the file to be deployed/undeployed
     * @param pOut output stream
     */
    private void deployJarModule(ObjectName onServer, Management mgmt, String jarFileName, PrintWriter pOut) {
        String[] signature = {"java.lang.String"};
        String[] params = new String[1];
        params[0] = jarFileName;
        pOut.println("<li>Test if " + jarFileName + " deployed</li>");
        boolean isDeployed = false;
        try {
            isDeployed = ((Boolean) mgmt.invoke(onServer, "isJarDeployed", params, signature)).booleanValue();
        } catch (Exception e) {
            pOut.println("<li>Problem when invoking isJarDeployed management operation: " + e.toString());
        }
        if (!isDeployed) {
            pOut.println("<li>Try to deploy " + jarFileName + " !</li>");
            try {
                String deployedObjectName = (String) mgmt.invoke(onServer, "deployJar", params, signature);
                pOut.println("<li>The Object Name of the deployed J2EEModule is: \"" + deployedObjectName + "\". </li>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not deploy " + jarFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not deploy " + jarFileName + " because of exception: " + e.toString() + " </li>");
            }
        } else {
            pOut.println("<li>Try to un-deploy " + jarFileName + " !</li>");
            try {
                mgmt.invoke(onServer, "unDeployJar", params, signature);
                pOut.println("<li>Done undeploy.>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not undeploy " + jarFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not undeploy " + jarFileName + " because of exception: " + e.toString() + " </li>");
            }
        }
    }
    /**
     * Deploy if not already deployed, undeploy otherwise, a given ear file.
     * @param onServer ObjectName of the J2EEServer MBean on which the mananagement operation has to be applied
     * @param mgmt MEJB
     * @param earFileName name of the file to be deployed/undeployed
     * @param pOut output stream
     */
    private void deployEarModule(ObjectName onServer, Management mgmt, String earFileName, PrintWriter pOut) {
        String[] signature = {"java.lang.String"};
        String[] params = new String[1];
        params[0] = earFileName;
        pOut.println("<li>Test if " + earFileName + " deployed</li>");
        boolean isDeployed = false;
        try {
            isDeployed = ((Boolean) mgmt.invoke(onServer, "isEarDeployed", params, signature)).booleanValue();
        } catch (Exception e) {
            pOut.println("<li>Problem when invoking isEarDeployed management operation: " + e.toString());
        }
        if (!isDeployed) {
            pOut.println("<li>Try to deploy " + earFileName + " !</li>");
            try {
                String deployedObjectName = (String) mgmt.invoke(onServer, "deployEar", params, signature);
                pOut.println("<li>The Object Name of the deployed J2EEModule is: \"" + deployedObjectName + "\". </li>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not deploy " + earFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not deploy " + earFileName + " because of exception: " + e.toString() + " </li>");
            }
        } else {
            pOut.println("<li>Try to un-deploy " + earFileName + " !</li>");
            try {
                mgmt.invoke(onServer, "unDeployEar", params, signature);
                pOut.println("<li>Done undeploy.>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not undeploy " + earFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not undeploy " + earFileName + " because of exception: " + e.toString() + " </li>");
            }
        }
    }

    /**
     * Deploy if not already deployed, undeploy otherwise, a given rar file.
     * @param onServer ObjectName of the J2EEServer MBean on which the mananagement operation has to be applied
     * @param mgmt MEJB
     * @param rarFileName name of the file to be deployed/undeployed
     * @param pOut output stream
     */
    private void deployRarModule(ObjectName onServer, Management mgmt, String rarFileName, PrintWriter pOut) {
        String[] signature = {"java.lang.String"};
        String[] params = new String[1];
        params[0] = rarFileName;
        pOut.println("<li>Test if " + rarFileName + " deployed</li>");
        boolean isDeployed = false;
        try {
            isDeployed = ((Boolean) mgmt.invoke(onServer, "isRarDeployed", params, signature)).booleanValue();
        } catch (Exception e) {
            pOut.println("<li>Problem when invoking isRarDeployed management operation: " + e.toString());
        }
        if (!isDeployed) {
            pOut.println("<li>Try to deploy " + rarFileName + " !</li>");
            try {
                String deployedObjectName = (String) mgmt.invoke(onServer, "deployRar", params, signature);
                pOut.println("<li>The Object Name of the deployed J2EEModule is: \"" + deployedObjectName + "\". </li>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not deploy " + rarFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not deploy " + rarFileName + " because of exception: " + e.toString() + " </li>");
            }
        } else {
            pOut.println("<li>Try to un-deploy " + rarFileName + " !</li>");
            try {
                mgmt.invoke(onServer, "unDeployRar", params, signature);
                pOut.println("<li>Done undeploy.>");
            } catch (MBeanException mbe) {
                pOut.println("<li>Could not undeploy " + rarFileName + " because of exception: "
                        + mbe.getTargetException().toString());
            } catch (Exception e) {
                pOut.println("<li>Could not undeploy " + rarFileName + " because of exception: " + e.toString() + " </li>");
            }
        }
    }
}